#!/usr/local/bin/perl
#
# global intialization.  one path name, default query info,  and some
# misc text strings.
#

#DOCUMENT_ROOT on this server doesn't already end with a slash.
$listdir = join("/",$ENV{"DOCUMENT_ROOT"},"cloister/pricelists/");

# default argument values: magic cards worth at least a hundred bucks.
%args = (
  "list",     "magic",
  "field",    "1",
  "operator", "ge",
  "target",   "100.00"
);

# text strings for making well-formed english sentences out of queries.
%games = (
  "magic", "Magic: the Gathering",
  "stccg", "Star Trek: the Collectable Card Game",
  "jyhad", "Jyhad/Vampire: the Eternal Struggle"
);
@fieldnames = ("card name", "price", "standard deviation", "average price",
               "high price", "low price", "price change since last week",
               "number of citations");
%opverbs = (
  "re", "matches the regular expression",
  "lt", "is less than",
  "le", "is less than or equal to",
  "eq", "equals",
  "ge", "is greater than or equal to",
  "gt", "is greater than"
);


#
# start off by outputting the preamble, so the user agent is less likely to 
# time out.  not that it's likely anyway, but still...
#
preamble();

#
# argument parsing:  puts the args into friendlier named variables, and builds
#                    a pathname to the database.
#

# get the query string from the server and split it into named args, and 
# shove them into an associative array
foreach (split("&",$ENV{"QUERY_STRING"}))
{
    ($name,$val) = split("=",$_);
    $args{$name} = $val;
}
$list     = $args{"list"};
$field    = $args{"field"};
$operator = $args{"operator"};
$target   = $args{"target"};

$listname = join("",$listdir,"current-$list"); # which list

#
# sanity checking and stuff:
#

# make sure the field in question is valid
if($field < 0 || $field > 7) {
  print "Error: invalid field.\n";
  postamble();
  exit;
}

# make sure the operator is valid
if(!($operator =~ /re|lt|le|eq|ge|gt/)) {
  print "Error: $operator is not valid.\n";
  print qq(Valid operators are: "re", "lt", "le", "eq", "ge", and "gt".\n);
  postamble();
  exit;
}

# ensure that checks against card names are done with case insensitive
# regular expression matches
if($field == 0) {
  $operator = "re";  # just to make sure...
  $target = unwebquote($target); # undo URL quoting of special characters
  $target = "(?i)\L$target\E";   # lowercase and insensitize the user's regexp.
} 

# make sure they're asking for a valid price database.  if not, give them a 
# list of values they can supply that yield valid databases.
if ((! -e $listname) || ($list =~ /\.gz|\.zip/)) {
  print qq(Error: no database for "$list".  Databases exist for:\n);

# glob all the filenames that might be price databases
  @games = glob(join("",$listdir,"current-*"));
  foreach (@games) {
    s/current-//;          # Strip the filenames down to just
    s/$listdir//;          # the part that they need to know so they don't
    print "$_\n";          # try to get clever by requesting the databases
  }                        # directly.

  postamble();
  exit;
}


#
# open the database and skip ahead to the useful part.
#
if(!open(PRICES,$listname)) { 
  print "error: could not open $list database\n";
  postamble();
  exit;
}
$junk = <PRICES>;  # throw away the top three lines--the timestamp and column
$junk = <PRICES>;  # headers--which are just there for the humans.
$junk = <PRICES>;  

#
# remind the user what their query is:
#

print "Query results for: $games{$list} cards where $fieldnames[$field] ";
print "$opverbs{$operator} $target\n";


#
# main loop: loop over each line in the database, and for each, see if the
# requested field fits the specified criteria.
#
while ($line = <PRICES>) { # read a line

  # parse the fields out of the line.  icky regexp, compiled only once.
  if( !(@fields = $line =~ m/([-\w,':\(\) ]+[\w':\(\)]),\s{2,}([\d.]+),\s*([\d.]+),\s*([\d.]+),\s*([\d.]+),\s*([\d.]+),\s*(-?[\d.]+),\s*(\d+)/o)) {
    next; # skip malformed lines, not that there should be any.
  }

  $match = 0; # this line is not a match yet.

# the if() statements in this block are intentionally nested so that perl can
# more easily optimize them internally into a switch() statement.
  SWITCH: {
    if($operator eq "re") {
      $fields[field] = "\L$fields[field]\E"; # make the card name lowecase
      if($fields[$field] =~ /$target/) {
        $match = 1; last SWITCH;
      }
    }
    if($operator eq "eq") {
      if ($fields[$field] == $target) {
        $match = 1; last SWITCH; 
      }
    }
    if($operator eq "lt") {
      if($fields[$field] < $target) {
        $match = 1; last SWITCH; 
      }
    }
    if($operator eq "gt") {
      if($fields[$field] > $target) {
        $match = 1; last SWITCH; 
      }
    }
    if($operator eq "le") {
      if($fields[$field] <= $target) {
        $match = 1; last SWITCH; 
      }
    }
    if($operator eq "ge") {
      if($fields[$field] >= $target) {
        $match = 1; last SWITCH; 
      }
    }
  }

# the payoff: if the line matched, print it out.
  if($match == 1) { 
    $matches++;
# nix the check that limits the search to 20 matches
#    if ($matches == 21) {
#      print "Your query matched more than 20 items.  Exiting.\n";
#      last;
#    }
    print $line;
  }
}

# all done.  clean up and exit
close(PRICES);
if($matches == 0) {
  print "No matches for your search criteria.\n";
}

postamble();
exit;

#
# support routines: preamble() and postamble() write the head and foot of the
# output html stream.
#

# write out the html header and set up the body
sub preamble {
print <<EOF
Content-type: text/html

  <html> <title>
  Cloister's Trading Card Price Lists--Search Results
  </title> <BODY> 
  <p><a href="retire.html">I'm Retiring</a>.  The pricelist for the week
  of June 28, 2000, will be the last one I publish.  Click on the link
  for details.</p>
  <PRE>
EOF
}

# write out the html footer
sub postamble {
print <<EOF
  </PRE>
  </BODY> </HTML>
EOF
}

# turn URL-quoted characters (spaces and many kinds of punctuation) from their
# quoted forms back into their real selves.
sub unwebquote {
  my $query = $_[0];

  $query =~ s/\+/ /g; # turn plusses into spaces

  while($query =~ m/%([0-9A-F]{2})/) { # turn punctuation back into itself.
    my $prestring = $`;
    my $poststring = $';
    $query = join("",$prestring,chr(hex($1)),$poststring);
  }
  return $query;
}
